@@ -890,6 +890,16 @@ class ParticleGroup:
         """Checks internal data"""
         return True if item in self._data else False    
     
+    def __eq__(self, other):
+        """Check equality of internal data"""
+        if isinstance(other, ParticleGroup):
+            for key in ['x', 'px', 'y', 'py', 'z', 'pz', 't', 'status', 'weight', 'id']:
+                if not np.all(self[key] == other[key]):
+                    return False
+            return True
+
+        return NotImplemented    
+    
     def __len__(self):
         return len(self[self._settable_array_keys[0]])
     
@@ -907,14 +917,14 @@ class ParticleGroup:
 # helper functions for ParticleGroup class
     
     
-def single_particle(x=0,
-                   px=0,
-                   y=0,
-                   py=0,
-                   z=0,
-                   pz=0,
-                   t=0,
-                   weight=1,
+def single_particle(x=0.0,
+                   px=0.0,
+                   y=0.0,
+                   py=0.0,
+                   z=0.0,
+                   pz=0.0,
+                   t=0.0,
+                   weight=1.0,
                    status=1,
                    species='electron'):
     """
